import Flutter
import UIKit

//支持ffi通道
struct NativeLogFlag : OptionSet {
    let rawValue : UInt
    static let error   = NativeLogFlag(rawValue: 1 << 0)
    static let warning  = NativeLogFlag(rawValue: 1 << 1)
    static let info  = NativeLogFlag(rawValue: 1 << 2)
    static let debug = NativeLogFlag(rawValue: 1 << 3)
    static let verbose = NativeLogFlag(rawValue: 1 << 4)
}
public typealias DartLogger = @convention(c) (_ flag: UInt,_ simulator: Int8,_ tag: UnsafePointer<CChar>,_ msg: UnsafePointer<CChar>) -> Void

@_cdecl("native_log_func")
public func native_log_func(flag : UInt, tag:UnsafePointer<CChar>, msg:UnsafePointer<CChar>, callback: DartLogger?) -> Int {
    guard let tagStr = String(utf8String: tag),
          let msgStr = String(utf8String: msg) else {
        print("tag or msg invalid")
        return -1
    }
    let args: [CVarArg] = [ tagStr, msgStr ]
    let logFlag = NativeLogFlag(rawValue: flag)
    var retcode = 0
    switch logFlag {
    case .verbose:
        withVaList(args) {
            DDLogWrapper.ddLogVerbose_("[flutter][%@] %@", args: $0)
        }
    case .debug:
        withVaList(args) {
            DDLogWrapper.ddLogDebug_("[flutter][%@] %@", args: $0)
        }
    case .info:
        withVaList(args) {
            DDLogWrapper.ddLogInfo_("[flutter][%@] %@", args: $0)
        }
    case .warning:
        withVaList(args) {
            DDLogWrapper.ddLogWarn_("[flutter][%@] %@", args: $0)
        }
    case .error:
        withVaList(args) {
            DDLogWrapper.ddLogError_("[flutter][%@] %@", args: $0)
        }
    default:
        print("level \(flag) invalid");
        retcode = -2
    }
    //callback to dart
    if let cbDart = callback {
        var simulator:Int8 = 0
#if targetEnvironment(simulator)
        simulator = 1
#endif
        cbDart(flag,simulator,tag,msg)
    }
    return retcode
}


//do not need
public class SwiftFlutterLoggerNativePlugin: NSObject, FlutterPlugin {
  public static func register(with registrar: FlutterPluginRegistrar) {
    // 不需要 methodchannel
  }
}
