import 'package:stack_trace/stack_trace.dart';

class LoggerTrace {
  late final String _callerName;

  String? fileName;
  int? lineNumber;
  int? columnNumber;

  LoggerTrace(this._callerName) {
    _parseTrace();
  }

  _parseTrace() {
    var chain = Chain.current(); // Chain.forTrace(StackTrace.current);
    // 将 core 和 flutter 包的堆栈合起来（即相关数据只剩其中一条）
    chain = chain.foldFrames((frame) => frame.isCore || frame.package == "flutter");
    // 取出所有信息帧
    final frames = chain.toTrace().frames;
    // 找到当前函数的信息帧
    final idx = frames.indexWhere((element) => element.member == _callerName);
    if (idx == -1 || idx+1 >= frames.length) {
      return;
    }
    // 调用当前函数的函数信息帧
    final frame = frames[idx+2];

    fileName =  frame.uri.toString();
    lineNumber =  frame.line;
    columnNumber =  frame.column;
  }
}
