import 'dart:ffi'; // For FFI
import 'dart:io'; // For Platform.isX
import 'package:ffi/ffi.dart';
import 'logger_trace.dart';
import 'logger_constants.dart';
import 'app_log_interface.dart';
import 'package:flutter/foundation.dart';

typedef _DartLogT = int Function(int,Pointer<Utf8>,Pointer<Utf8>,Pointer?);
typedef _NativeLogT = Int32 Function(Uint32,Pointer<Utf8>,Pointer<Utf8>,Pointer?);
typedef _NativeCallbackT = Void Function(Uint32,Int8,Pointer<Utf8>,Pointer<Utf8>);

//通过FFI实现
class LogFFI extends AppLogInterface{
  static final DynamicLibrary _nativeLogLib = Platform.isAndroid
      ? DynamicLibrary.open("libnative_log.so")
      : DynamicLibrary.process();

  final _nativeLogFunc = _nativeLogLib
      .lookup<NativeFunction<_NativeLogT>>("native_log_func")
      .asFunction<_DartLogT>();

  final _wrappedPrintPointer = Pointer.fromFunction<_NativeCallbackT>(_wrappedPrint);

  //单例
  static LogFFI? _instance;
  factory LogFFI() {
    if(_instance == null) {
      _instance = LogFFI._();
    }
    return _instance!;
  }
  LogFFI._();

  //public method
  void e(Object obj) {
    _ffi_Log(YDLogFlag.Error,"Error",obj);
  }
  void w(Object obj) {
    _ffi_Log(YDLogFlag.Warning,"Warning",obj);
  }
  void i(Object obj) {
    _ffi_Log(YDLogFlag.Info,"Info",obj);
  }
  void d(Object obj) {
    _ffi_Log(YDLogFlag.Debug,"Debug",obj);
  }
  void v(Object obj) {
    _ffi_Log(YDLogFlag.Verbose,"Verbose",obj);
  }

  //private method
  void _ffi_Log(int flag,String tag,Object? obj){
    if(0 == (flag & currentLevel)){
      return;
    }

    LoggerTrace programInfo = LoggerTrace("LogFFI._ffi_Log");
    String tagstr = "${DateTime.now()} ${tag}-> ${programInfo.fileName}_${programInfo.lineNumber}";
    String msg = obj.toString();

    var tagPointer = tagstr.toNativeUtf8();
    var msgPointer = msg.toNativeUtf8();
    var callback = kDebugMode?_wrappedPrintPointer:nullptr;
    var result = _nativeLogFunc(flag,tagPointer,msgPointer,callback);
    if(kDebugMode){
      assert(result==0, "log failed:$result");
    }
    malloc.free(tagPointer);
    malloc.free(msgPointer);
  }

  static void _wrappedPrint(int flag,int simulator,Pointer<Utf8> tag,Pointer<Utf8> msg){
    var tagstr = tag.toDartString();
    var msgstr = msg.toDartString();
    if(simulator != 0){
      print("$tagstr, $msgstr");
    }
  }
}